package java.time.temporal;

import java.io.IOException;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.DateTimeException;
import java.io.Serializable;

public final class ValueRange implements Serializable
{
    private static final long serialVersionUID = -7317881728594519368L;
    private final long minSmallest;
    private final long minLargest;
    private final long maxSmallest;
    private final long maxLargest;
    
    public static ValueRange of(long n, long n2) {
        if (n > n2) {
            throw new IllegalArgumentException("Minimum value must be less than maximum value");
        }
        return new ValueRange(n, n, n2, n2);
    }
    
    public static ValueRange of(long n, long n2, long n3) {
        return of(n, n, n2, n3);
    }
    
    public static ValueRange of(long n, long n2, long n3, long n4) {
        if (n > n2) {
            throw new IllegalArgumentException("Smallest minimum value must be less than largest minimum value");
        }
        if (n3 > n4) {
            throw new IllegalArgumentException("Smallest maximum value must be less than largest maximum value");
        }
        if (n2 > n4) {
            throw new IllegalArgumentException("Minimum value must be less than maximum value");
        }
        return new ValueRange(n, n2, n3, n4);
    }
    
    private ValueRange(long minSmallest, long minLargest, long maxSmallest, long maxLargest) {
        this.minSmallest = minSmallest;
        this.minLargest = minLargest;
        this.maxSmallest = maxSmallest;
        this.maxLargest = maxLargest;
    }
    
    public boolean isFixed() {
        return this.minSmallest == this.minLargest && this.maxSmallest == this.maxLargest;
    }
    
    public long getMinimum() {
        return this.minSmallest;
    }
    
    public long getLargestMinimum() {
        return this.minLargest;
    }
    
    public long getSmallestMaximum() {
        return this.maxSmallest;
    }
    
    public long getMaximum() {
        return this.maxLargest;
    }
    
    public boolean isIntValue() {
        return this.getMinimum() >= -2147483648L && this.getMaximum() <= 2147483647L;
    }
    
    public boolean isValidValue(long n) {
        return n >= this.getMinimum() && n <= this.getMaximum();
    }
    
    public boolean isValidIntValue(long n) {
        return this.isIntValue() && this.isValidValue(n);
    }
    
    public long checkValidValue(long n, TemporalField temporalField) {
        if (!this.isValidValue(n)) {
            throw new DateTimeException(this.genInvalidFieldMessage(temporalField, n));
        }
        return n;
    }
    
    public int checkValidIntValue(long n, TemporalField temporalField) {
        if (!this.isValidIntValue(n)) {
            throw new DateTimeException(this.genInvalidFieldMessage(temporalField, n));
        }
        return (int)n;
    }
    
    private String genInvalidFieldMessage(TemporalField obj, long n) {
        if (obj != null) {
            return "Invalid value for " + obj + " (valid values " + this + "): " + n;
        }
        return "Invalid value (valid values " + this + "): " + n;
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException, InvalidObjectException {
        objectInputStream.defaultReadObject();
        if (this.minSmallest > this.minLargest) {
            throw new InvalidObjectException("Smallest minimum value must be less than largest minimum value");
        }
        if (this.maxSmallest > this.maxLargest) {
            throw new InvalidObjectException("Smallest maximum value must be less than largest maximum value");
        }
        if (this.minLargest > this.maxLargest) {
            throw new InvalidObjectException("Minimum value must be less than maximum value");
        }
        return;
    }
    
    @Override
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if ((o instanceof ValueRange)) {
            ValueRange valueRange = (ValueRange)o;
            return this.minSmallest == valueRange.minSmallest && this.minLargest == valueRange.minLargest && this.maxSmallest == valueRange.maxSmallest && this.maxLargest == valueRange.maxLargest;
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        long n = this.minSmallest + (this.minLargest << 16) + (this.minLargest >> 48) + (this.maxSmallest << 32) + (this.maxSmallest >> 32) + (this.maxLargest << 48) + (this.maxLargest >> 16);
        return (int)(n ^ n >>> 32);
    }
    
    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append(this.minSmallest);
        if (this.minSmallest != this.minLargest) {
            sb.append('/').append(this.minLargest);
        }
        sb.append(" - ").append(this.maxSmallest);
        if (this.maxSmallest != this.maxLargest) {
            sb.append('/').append(this.maxLargest);
        }
        return sb.toString();
    }
}
